from agenda_culturel.models import Event
import json
from datetime import datetime


class EventsImporter:

    def __init__(self, celery_id):
        self.celery_id = celery_id
        self.error_message = ""
        self.init_result_properties()

    def init_result_properties(self):
        self.event_objects = []
        self.db_event_objects = []
        self.nb_updated = 0
        self.nb_removed = 0
        self.date = None
        self.url = None

    def get_nb_events(self):
        return len(self.event_objects)

    def get_nb_imported_events(self):
        return len(self.db_event_objects)

    def get_nb_updated_events(self):
        return self.nb_updated

    def get_nb_removed_events(self):
        return self.nb_removed

    def import_events(self, json_structure):
        self.init_result_properties()

        try:
            structure = json.loads(json_structure)
        except:
            return (False, "JSON file is not correctly structured")

        if not "header" in structure:
            return (False, "JSON is not correctly structured: missing header")
        if not "events" in structure:
            return (False, "JSON is not correctly structured: missing events")

        if "url" in structure["header"]:
            self.url = structure["header"]["url"]
        else:
            return (False, "JSON is not correctly structured: missing url in header")

        if "date" in structure["header"]:
            self.date = structure["header"]["date"]

        # get events
        for event in structure["events"]:
            if "created_date" not in event:
                if self.date is not None:
                    event["created_date"] = self.date
                else:
                    event["created_date"] = datetime.now()

            if not self.import_event(event):
                return (False, self.error_message)
        
        # import them
        self.save_imported()

        return (True, "")

    def save_imported(self):
        self.db_event_objects, self.nb_updated, self.nb_removed = Event.import_events(self.event_objects, remove_missing=True)

    
    def is_valid_event_structure(self, event):
        if "title" not in event:
            self.error_message = "JSON is not correctly structured: one event without title"
            return False
        if "start_day" not in event:
            self.error_message = "JSON is not correctly structured: one event without start_day"
            return False
        return True


    def import_event(self, event):
        if self.is_valid_event_structure(event):
            event_obj = Event.from_structure(event)
            self.event_objects.append(event_obj)
            return True
        else:
            return False


